@@ -15,7 +15,8 @@ from django.db.models import Q
 from xj_user.services.user_service import UserService
 from ..models import UserToGroup, RoleUserGroup, UserToRole
 from ..services.role_service import RoleService
-from ..utils.custom_tool import format_params_handle, parse_json, format_list_handle
+from ..utils.custom_tool import format_params_handle, parse_json, format_list_handle, force_transform_type, \
+    filter_fields_handler
 from ..utils.j_recur import JRecur
 from ..utils.join_list import JoinList
 
@@ -39,6 +40,8 @@ class UserGroupService(object):
         res = RoleUserGroup.objects.filter(group=params.get("group")).exists()
         if res:
             return None, "组标签，必须唯一"
+        if params.get("parent_group_id") == 0:
+            params.pop("parent_group_id")
         instance = RoleUserGroup.objects.create(**params)
         return None, None
 
@@ -66,6 +69,26 @@ class UserGroupService(object):
             instance.update(**params)
         return None, None
 
+    @staticmethod
+    def details_group(params: dict):
+        """
+        用户分组编辑，可编辑参数："group", "group_name", "parent_group_id", "description"
+        :param params: 编辑参数
+        :return: None,err
+        """
+        params = format_params_handle(
+            param_dict=params,
+            filter_filed_list=["id", "group", "group_name", "parent_group_id", "description"]
+        )
+        id = params.pop("id", None)
+        if not id:
+            return None, "ID 不可以为空"
+        instance = RoleUserGroup.objects.filter(id=id).first()
+        if not instance:
+            return None, "分组不存在"
+        instance = instance.to_json()
+        return instance, None
+
     @staticmethod
     def del_group(id: int):
         """
@@ -84,31 +107,74 @@ class UserGroupService(object):
         return None, None
 
     @staticmethod
-    def group_list(params):
+    def group_list(params: dict, need_pagination: bool = True, only_first: bool = False, filter_fields: bool = None,
+                   **kwargs):
         """
         用户的分组列表，分页查询。
+        :param filter_fields: 过滤字段
+        :param only_first: 仅仅查询第一条
+        :param need_pagination: 是否需要分页查询
         :param params: 请求的
         :return: page_set,err
         """
-        page = params.pop("page", 1)
-        size = params.pop("size", 20)
+        kwargs, is_pass = force_transform_type(variable=kwargs, var_type="dict", default={})
+        params, is_pass = force_transform_type(variable=params, var_type="dict", default={})
+        params.update(kwargs)
+        need_pagination, is_pass = force_transform_type(variable=need_pagination, var_type="bool", default=True)
+        only_first, is_pass = force_transform_type(variable=only_first, var_type="bool", default=False)
+        page, is_pass = force_transform_type(variable=params.pop("page", 1), var_type="int", default=1)
+        size, is_pass = force_transform_type(variable=params.pop("size", 10), var_type="int", default=10)
+        sort = params.pop("sort", "-user_group_id")
+        sort = sort if sort in ["user_group_id", "-user_group_id", "sort", "-sort"] else "-user_group_id"
+
+        user_id = params.pop("user_id", None)
+        user_id_list = params.pop("user_id_list", None)
+
         # 参数有效性处理
         params = format_params_handle(
             param_dict=params,
-            filter_filed_list=["group", "group_name", "parent_group_id"],
-            alias_dict={"group_name": "group_name__contains"}
+            filter_filed_list=["id|int", "user_group_id|int", "user_id|int", "user_id_list|list_int",
+                               "id_list|list_int", "group", "group_name", "parent_group_id|int"],
+            alias_dict={"group_name": "group_name__contains", "group": "group__contains", "user_id_list": "user_id__in",
+                        "id_list": "id__in"},
+            split_list=["user_id_list", "id_list"]
         )
-        # sql 查询数据
-        group_set = RoleUserGroup.objects.filter(**params)
-        count = group_set.count()
-        group_list = group_set.values()
+        all_field_list = ["id", "user_group_id", "group", "group_name", "parent_group_id", "sort", "description"]
+        filter_fields = filter_fields_handler(input_field_expression=filter_fields, all_field_list=all_field_list)
+
+        # sql查询数据
+        if not (user_id or user_id_list):
+            group_set = RoleUserGroup.objects.annotate(user_group_id=F("id")).filter(**params)
+        else:
+            filter_fields.append("user_id")  # 返回用户ID
+            group_set = UserToGroup.objects.annotate(
+                group=F("user_group__group"),
+                group_name=F("user_group__group_name"),
+                parent_group_id=F("user_group__parent_group_id"),
+                description=F("user_group__description"),
+            ).filter(**params)
+
+        group_set = group_set.order_by(sort)
+        group_list = group_set.values(*filter_fields)
+
+        # 仅仅返回第一条
+        if only_first:
+            return group_list.first(), None
+
+        # 返回全部，不分页
+        total = group_set.count()
+        if (not need_pagination and total <= 200) or (user_id or user_id_list):
+            return list(group_set), None
+
+        # 分页查询
         paginator = Paginator(group_list, size)
         try:
             finish_set = paginator.page(page)
         except EmptyPage:
-            finish_set = paginator.page(paginator.num_pages)
+            return {"page": int(page), "size": int(size), "count": int(total), "total": total, "list": []}, None
 
-        return {"page": int(page), "size": int(size), "count": int(count), "list": list(finish_set.object_list)}, None
+        return {"page": int(page), "size": int(size), "count": int(total), "total": total,
+                "list": list(finish_set.object_list)}, None
 
     @staticmethod
     def get_user_ids_by_group(group_id):
@@ -145,7 +211,8 @@ class UserGroupService(object):
             is_remove_empty=True
         )
         if user_search_params:
-            user_serv, err = UserService.user_list(params=user_search_params, need_Pagination=False, is_strict_mode=False)
+            user_serv, err = UserService.user_list(params=user_search_params, need_Pagination=False,
+                                                   is_strict_mode=False)
             params["user_id_list"] = [i["user_id"] for i in user_serv if i]
 
         # 是否需要子节点搜索
@@ -161,33 +228,37 @@ class UserGroupService(object):
         # 分组下面用户分页查询
         group_search_params = format_params_handle(
             param_dict=params,
-            filter_filed_list=["group_id|int", "user_group_id|int", "user_group_id_list|list", "user_id|int", "user_id_list|list"],
-            alias_dict={"user_group_id_list": "user_group_id__in", "group_id": "user_group_id", "user_id_list": "user_id__in"},
+            filter_filed_list=["group_id|int", "user_group_id|int", "user_group_id_list|list", "user_id|int",
+                               "user_id_list|list"],
+            alias_dict={"user_group_id_list": "user_group_id__in", "group_id": "user_group_id",
+                        "user_id_list": "user_id__in"},
         )
 
         # 分组用户查询
         user_group_set = UserToGroup.objects.filter(**group_search_params).annotate(
-            user_group_value=F("user_group__group_name")
-        ).values("user_id", "user_group_value")
+            user_group_value=F("user_group__group"),
+            user_group_name=F("user_group__group_name")
+        ).values("user_id", "user_group_name", "user_group_value")
         total = user_group_set.count()
         paginator = Paginator(user_group_set, size)
         # 分页保护
         try:
             group_user_list = list(paginator.page(page).object_list)
         except EmptyPage:
-            group_user_list = list(paginator.page(paginator.num_pages).object_list)
+            return {"total": total, "page": int(page), "size": int(size), "list": []}, None
 
         # 拼接用户信息
-        user_serv, err = UserService.user_list(allow_user_list=list(set([it['user_id'] for it in group_user_list])), need_Pagination=False)
+        user_serv, err = UserService.user_list(allow_user_list=list(set([it['user_id'] for it in group_user_list])),
+                                               need_Pagination=False)
         group_user_list = JoinList(l_list=group_user_list, r_list=user_serv, l_key="user_id", r_key="user_id").join()
 
         # 由于数据迁移导致部门绑定的用户ID,用户表存在该用户的信息了,遍历做出提示
         for i in group_user_list:
             if i.get("user_name", None) is None:
+                i["full_name"] = "--- 该用户不存在,请移除.然后重新绑定 ---"
                 i["email"] = ""
-                i["full_name"] = "该用户不存在,请移除.然后重新绑定."
-                i["user_name"] = "该用户不存在,请移除.然后重新绑定."
-                i["nickname"] = "该用户不存在,请移除.然后重新绑定."
+                i["user_name"] = ""
+                i["nickname"] = ""
                 i["register_time"] = ""
                 i["wechat_openid"] = ""
 
@@ -281,12 +352,16 @@ class UserGroupService(object):
             return None, str(e)
 
     @staticmethod
-    def get_user_group_tree(group_id=None, is_family_tree=False):
+    def get_user_group_tree(group_id=None, is_family_tree=False, params: dict = None):
         """
         用戶分组树
         @param group_id 用户组ID，如果有则过滤用户组，没有会返回全部分组，请慎用。
         @param is_family_tree 是否返回整个家族数。
+        @param params: 过滤树参数
         """
+        params, err = force_transform_type(variable=params, var_type="only_dict", default={})
+        filter_tree_params, err = JRecur.get_filter_tree_params(params=params)
+
         data_list = list(RoleUserGroup.objects.filter().annotate(name=F('group_name')).values(
             "id",
             "group",
@@ -302,6 +377,13 @@ class UserGroupService(object):
                 find_key='id', find_value=group_id,
                 is_family_tree=is_family_tree
             )
+
+        elif filter_tree_params:
+            for k, v in filter_tree_params.items():
+                if not v:
+                    continue
+                group_tree = JRecur.filter_forest(group_tree, find_key=k, find_value=v, children_key='children')
+
         return group_tree, None
 
     @staticmethod
@@ -316,7 +398,10 @@ class UserGroupService(object):
         if err:
             return None, err
         # 角色列表，并建立映射
-        role_list, err = RoleService.get_role_list({}, None)
+        role_list, err = RoleService.get_role_list(
+            params={},
+            need_pagination=False
+        )
         role_dict = {}
         for role in role_list:
             index = str(role['user_group_id'])
@@ -337,6 +422,7 @@ class UserGroupService(object):
                     for i in role_list:
                         i["name"] = i.get("role_name", "")
                         i["type"] = "role"
+                        i["id"] = i.get("role_id")
                     item['children'].extend(role_dict[str(item['id'])])
                 item["name"] = item.get("group_name", "")
                 item["type"] = "group"
@@ -469,8 +555,8 @@ class UserGroupService(object):
     @staticmethod
     def group_user_edit(params):  # 更名或者单独起一个文件 方法这个方法
         # 用户角色部门绑定
-        user_role_list = params.get('user_role_id_list', None)
-        user_group_list = params.get('user_group_id_list', None)
+        user_role_list = params.get('user_role_list', None)
+        user_group_list = params.get('user_group_list', None)
         user_id = params.get("user_id")
         if not user_id:
             return None, "msg:用户ID错误;tip:非法请求"
@@ -480,7 +566,7 @@ class UserGroupService(object):
                 return None, "msg:" + str(err).replace(":", " ") + ";tip:部门修改异常"
 
         if not user_role_list is None:
-            data, err = RoleService.bind_user_role(params.get("user_id"), user_role_list)
+            data, err = RoleService.bind_user_roles(params.get("user_id"), user_role_list)
             if err:
                 return None, "msg:" + str(err).replace(":", " ") + ";tip:角色修改异常"
 
